#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import my_path
import pprint
import algobroker
from algobroker import Broker
from cryptoexchange import bitmex

class BrokerBitmex(Broker):
    def __init__(self):
        Broker.__init__(self, "broker_bitmex")
        self.api = None
        self.src_number = None
        self.dst_number = None
    def process_data(self, data):
        if self.api == None or \
           self.src_number == None or \
           self.dst_number == None:
            self.error("keys not initialized")
        if (data['action'] == "alert" and \
            data['type'] == 'sms'):
            params = {
                'src' : self.src_number,
                'dst' : self.dst_number[data['dst']],
                'text' : data['text'],
                'method' : 'POST'
                }
            self.debug(pprint.pformat(params))
            response = self.api.send_message(params)
            self.debug(pprint.pformat(str(response)))
        else:
            self.error("unknown item")
            self.error(pprint.pformat(data))
    def process_control(self, data):
        self.info("received control message")
        try:
            if data['cmd'] == "auth":
                self.auth_id = data['PLIVO_AUTH_ID']
                self.auth_token = data['PLIVO_AUTH_TOKEN']
                self.api = plivo.RestAPI(self.auth_id, self.auth_token)
                self.src_number = data['src_number']
                self.dst_number = data['dst_number']
        except:
            self.error("error processing control message")

if __name__ == "__main__":
    bp = BrokerBitmex()
    bp.run()



# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import logging
import zmq
import msgpack
import time

def logger(s : str):
    logger = logging.getLogger(s)
    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    formatter.converter = time.gmtime
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    return logger

data_ports = {
    "dispatcher" : "tcp://127.0.0.1:5557",
    "broker_plivo" : "tcp://127.0.0.1:5558",
    "strategy_alert" : "tcp://127.0.0.1:5559",
    "ticker_yahoo" : "tcp://127.0.0.1:5560",
    "ticker_bitfutures" : "tcp://127.0.0.1:5561",
    "broker_bitmex" : "tcp://127.0.0.1:5562"
    }

control_ports = {
    "dispatcher" : "tcp://127.0.0.1:5577",
    "broker_plivo" : "tcp://127.0.0.1:5578",
    "strategy_alert" : "tcp://127.0.0.1:5579",
    "ticker_yahoo" : "tcp://127.0.0.1:5580",
    "ticker_bitfutures" : "tcp://127.0.0.1:5581",
    "broker_bitmex" : "tcp://127.0.0.1:5582"
    }

def send_control(name, data):
    context = zmq.Context()
    socket = context.socket(zmq.PUSH)
    socket.connect(control_ports[name])
    socket.send(msgpack.packb(data))

class AlgoObject(object):
    def __init__(self, name : str, socket_type):
        self._logger = logger(name)
        self._context = zmq.Context()
        self._poller = zmq.Poller()
        self._data_socket = self.socket(socket_type)

        self._control_socket = self.socket(zmq.PULL)
        self._control_socket.bind(control_ports[name])
        self._poller.register(self._data_socket,
                              zmq.POLLIN)
        self._poller.register(self._control_socket,
                              zmq.POLLIN)
        self.info("starting %s" % name)
        self.timeout = None
    def socket(self, socket_type):
        return self._context.socket(socket_type)
    def send_data(self, message):
        self._data_socket.send(msgpack.packb(message))
    def recv_data(self):
        return msgpack.unpackb(self._data_socket.recv(),
                               encoding='utf-8')
    def recv_control(self):
        return msgpack.unpackb(self._control_socket.recv(),
                               encoding='utf-8')
    def debug(self, s):
        self._logger.debug(s)
    def info(self, s):
        self._logger.info(s)
    def error(self, s):
        self._logger.error(s)
    def warning(self, s):
        self._logger.warning(s)
    def process_data(self, data):
        raise NotImplementedError
    def process_control(self, data):
        pass
    def run_once(self):
        pass
    def run(self):
        while True:
            try:
                socks = dict(self._poller.poll(self.timeout))
            except KeyboardInterrupt:
                break
            if self._control_socket in socks:
                control = self.recv_control()
                self.process_control(control)
            if self._data_socket in socks:
                data = self.recv_data()
                self.process_data(data)
            self.run_once()
            

class Broker(AlgoObject):
    def __init__(self, name):
        AlgoObject.__init__(self, name, zmq.PULL)
        self._data_socket.bind(data_ports[name])



# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import time
import zmq
import algobroker
import msgpack

def client():
    context = zmq.Context()
    zmq_socket = context.socket(zmq.PUSH)
    zmq_socket.bind(algobroker.ports.dispatcher)
    # Start your result manager and workers before you start your
    work_message = { 'action' : 'log',
                     'item' : 'hello' }
    zmq_socket.send(msgpack.packb(work_message))
    work_message = { 'action' : 'alert',
                     'type' : 'sms',
                     'dst'  : 'trader2',
                     'text' : 'hello and happy trading' }
    zmq_socket.send(msgpack.packb(work_message))

#    zmq_socket = context.socket(zmq.PUSH)
#    zmq_socket.bind(algobroker.ports.plivo)
#    work_message = { 'action' : 'alert',
#                     'type' : 'sms',
#                     'dst'  : 'trader1',
#                     'text' : 'hello' }
#    zmq_socket.send(msgpack.packb(work_message))
client()



#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License
import my_path
import time
import zmq
import algobroker
import msgpack

from algobroker.quote_monitor import QuoteMonitor

if __name__ == "__main__":
    qm = QuoteMonitor()
    qm.test1()



#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import my_path
import time
import zmq
import pprint
import algobroker
import msgpack
from algobroker import Broker

class Dispatcher(Broker):
    def __init__(self):
        Broker.__init__(self, "dispatcher")
        # send work
        self.sms_sender = self.socket(zmq.PUSH)
        self.sms_sender.connect(algobroker.data_ports['broker_plivo'])
    def process_data(self, data):
        if (data['action'] == "log"):
            self.warning(pprint.pformat(data))
        elif (data['action'] == 'alert' and \
              data['type'] == 'sms'):
            self.debug("sending sms")
            self.debug(pprint.pformat(data))
            self.sms_sender.send(msgpack.packb(data))
        else:
            self.error("unknown action")

if __name__ == "__main__":
    dispatcher = Dispatcher()
    dispatcher.run()




#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import my_path
import time
import zmq
import algobroker
from algobroker import AlgoObject
import msgpack
import pprint
from yahoo_finance import Share

class YahooTicker(AlgoObject):
    def __init__(self):
        AlgoObject.__init__(self, "ticker_yahoo", zmq.PUB)
        self._data_socket.bind(algobroker.data_ports["ticker_yahoo"])
        self.time_limits = {}
        self.state = {}
        self.prev_state = {}
        self.assets = []
        self.quotes = {}
        self.timeout = 30000
        self.sleep = 30
        self.maintainence = 60 * 30
    def get_quotes(self):
        self.debug("getting quotes")
        try:
            for i in self.assets:
                yahoo = Share(i)
                self.quotes[i] = {
                    "source" : "yahoo",
                    "last" : float(yahoo.get_price())
                    }
        except OSError:
            self.error("Network Error")
    def send_quotes(self):
        self.debug("Sending quotes")
        self.send_data(self.quotes)
    def test(self):
        self.get_quotes()
        socket = self._context.socket(zmq.PUSH)
        socket.bind(algobroker.ports.dispatcher)
        message = { 'action' : 'log',
                    'item' : self.quotes }
        self._logger.debug("Sending data")
        socket.send(msgpack.packb(message))
    def process_control(self, data):
        self.debug("received control message")
        if data['cmd'] == "set":
            if 'assets' in data:
                self.debug("setting asset list")
                self.debug(pprint.pformat(data['assets']))
                self.assets = data['assets']
    def run_once(self):
        self.debug("running loop function")
        self.get_quotes()
        self.send_quotes()

if __name__ == "__main__":
    yq = YahooTicker()
    yq.run()



#!/usr/bin/python3
import algobroker
algobroker.send_control("ticker_yahoo",
                       {	
    "cmd" : "set",     
    "assets" : ["3888.HK", "0700.HK", "0388.HK"]
    })

algobroker.send_control("ticker_bitfutures",
                       {	
    "cmd" : "set",
    "exchanges" : ["bitmex"]
    })

algobroker.send_control("strategy_alert",
                        {	
    "cmd" : "set",
    "limits" : {
    "3888.HK" : [ 14.8, 15.5],	
    "0700.HK" : [ 125.0, 130.0],
    "0388.HK" : [ 175.0, 180.0]
    }
    })
